//
//  CommunityEditorViewModel.swift
//  SampleApp
//
//  Created by Michael Abadi on 13/07/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

extension AmityCommunityPostSettings {
    
    var title: String {
        switch self {
        case .onlyAdminCanPost:
            return "ONLY ADMIN CAN POST"
        case .adminReviewPostRequired:
            return "ADMIN REVIEW POST REQUIRED"
        case .anyoneCanPost:
            return "ANYONE CAN POST"
        @unknown default:
            return ""
        }
    }
    
}

enum CommunityStorySettings: Int, CaseIterable {
    
    case allowComment
    case none
    
    var title: String {
        switch self {
        case .allowComment:
            return "ALLOW COMMENT ON STORY"
        case .none:
            return "None"
        }
    }
    
    static func convertFromAmitySettings(_ settings: AmityCommunityStorySettings) -> CommunityStorySettings {
        let isAllowCommentOnStory = settings.allowComment
        if isAllowCommentOnStory {
            return .allowComment
        } else {
            return .none
        }
    }
    
}

struct CommunityDraft {
    
    internal init(identifier: String?, isPrivate: Bool, displayName: String, description: String, userIds: [String], tags: [String], key: String, value: String, membersCount: Int = 0, categoryId: String, postSettings: AmityCommunityPostSettings?, storySettings: CommunityStorySettings?) {
        self.identifier = identifier
        self.isPrivate = isPrivate
        self.displayName = displayName
        self.description = description
        self.userIds = userIds
        self.tags = tags
        self.key = key
        self.value = value
        self.membersCount = membersCount
        self.categoryId = categoryId
        self.postSettings = postSettings
        self.storySettings = storySettings
    }
    
    let identifier: String?
    var isPrivate: Bool
    var displayName: String
    var description: String
    var userIds: [String]
    var tags: [String]
    var key: String
    var value: String
    var membersCount: Int = 0
    var categoryId: String
    var postSettings: AmityCommunityPostSettings? {
        didSet {
            print("-> \(postSettings?.title ?? "")")
        }
    }
    var storySettings: CommunityStorySettings? {
        didSet {
            print("-> \(storySettings?.title ?? "")")
        }
    }
    
    var metadata: [String: String]? {
        if key == "" || value == "" {
            return nil
        }
        return [
            key: value
        ]
    }
    
    init(community: CommunityModel) {
        self = CommunityDraft(identifier: community.id, isPrivate: !community.isPublic, displayName: community.displayName, description: community.description, userIds: [community.userId], tags: community.tags, key: community.metadata?.keys.first ?? "", value: community.metadata?.description ?? "", membersCount: community.membersCount, categoryId: community.categoryIds?.first ?? "", postSettings: community.postSettings, storySettings: CommunityStorySettings.convertFromAmitySettings(community.storySettings))
    }
}

protocol CommunityEditorDatasource {
    var draft: CommunityDraft { get set }
    var isEditorLoading: Bool { get }
    var isEditMode: Bool { get }
    var shouldShowImagePicker: Bool { get set }
    var selectedImage: UIImage? { get set}
}

protocol CommunityEditorAction {
    func createCommunity(completion: @escaping (_ success: Bool, _ error: Error?) -> Void)
    func updateCommunity(completion: @escaping (_ success: Bool, _ error: Error?) -> Void)
    func deleteCommunity(completion: @escaping (_ success: Bool, _ error: Error?) -> Void)
}

// Base model protocol
protocol EditorViewModel: ObservableObject {
    var action: CommunityEditorAction { get }
    var datasource: CommunityEditorDatasource { get set }
}

// Concrete implementor
class CommunityEditorViewModel: EditorViewModel, CommunityEditorDatasource, CommunityEditorAction {
        
    private let communityRepository = AmityCommunityRepository(client: AmityManager.shared.client!)
    private let fileRepository = AmityFileRepository(client: AmityManager.shared.client!)
    
    @Published var draft: CommunityDraft = CommunityDraft(identifier: nil, isPrivate: false, displayName: "", description: "", userIds: [], tags: [], key: "", value: "", categoryId: "", postSettings: nil, storySettings: nil)
    @Published var isEditorLoading: Bool = false
    @Published var isEditMode: Bool = false
    @Published var shouldShowImagePicker: Bool = false
    @Published var selectedImage: UIImage?
    
    init(draft: CommunityDraft = CommunityDraft(identifier: nil, isPrivate: false, displayName: "", description: "", userIds: [], tags: [], key: "", value: "", categoryId: "", postSettings: nil, storySettings: nil)) {
        self.draft = draft
        if draft.identifier != nil {
            isEditMode = true
        }
    }
    
    func uploadAvatar(image: UIImage, completion:@escaping (AmityImageData?) -> Void) {
        Task { @MainActor in
            do {
                let result = try await fileRepository.uploadImage(image, progress: nil)
                completion(result)
            } catch {
                Log.add(info: "Upload Image Error: \(error.localizedDescription)")
            }
        }
    }
    
    func createCommunity(completion: @escaping (Bool, Error?) -> Void) {
        isEditorLoading = true
        getAmityImageData { [weak self] amityImageData in
            self?.createCommunityWithBuilder(imageData: amityImageData, completion: completion)
        }
    }
    
    func createCommunityWithBuilder(imageData: AmityImageData?, completion: @escaping (Bool, Error?) -> Void) {
        Task { @MainActor in
            let builder = AmityCommunityCreateOptions()
            builder.setDisplayName(draft.displayName)
            var finalUserId = draft.userIds.compactMap({$0})
            if !finalUserId.contains(AmityManager.shared.client!.currentUserId!) {
                finalUserId.append(AmityManager.shared.client!.currentUserId!)
            }
            builder.setUserIds(finalUserId)
            builder.setCommunityDescription(draft.description)
            builder.setIsPublic(!draft.isPrivate)
            if let postSettings = draft.postSettings {
                builder.setPostSettings(postSettings)
            }
            if let storySettings = draft.storySettings {
                builder.setStorySettings(allowComment: storySettings == .allowComment)
            }
            if let data = imageData {
                builder.setAvatar(data)
            }
            
            if !draft.categoryId.isEmpty {
                builder.setCategoryIds([draft.categoryId])
            }
            
            if draft.metadata != nil {
                builder.setMetadata(draft.metadata!)
            }
            
            do {
                let _ = try await communityRepository.createCommunity(with: builder)
                self.isEditorLoading = false
                completion(true, nil)
            } catch {
                self.isEditorLoading = false
                completion(true, error)
            }
        }
    }
    
    func updateCommunity(completion: @escaping (Bool, Error?) -> Void) {
        isEditorLoading = true
        getAmityImageData { [weak self] amityImageData in
            self?.updateCommunityWithBuilder(imageData: amityImageData, completion: completion)
        }
    }
    
    func updateCommunityWithBuilder(imageData: AmityImageData?, completion: @escaping (Bool, Error?) -> Void) {
        Task { @MainActor in
            let builder = AmityCommunityUpdateOptions()
            builder.setDisplayName(draft.displayName)
            builder.setCommunityDescription(draft.description)
            builder.setIsPublic(!draft.isPrivate)
            builder.setAvatar(imageData)
            if let postSettings = draft.postSettings {
                builder.setPostSettings(postSettings)
            }
            if let storySettings = draft.storySettings {
                builder.setStorySettings(allowComment: storySettings == .allowComment)
            }
            if !draft.categoryId.isEmpty {
                builder.setCategoryIds([draft.categoryId])
            }
            
            if draft.metadata != nil {
                builder.setMetadata(draft.metadata!)
            }
            
            do {
                let community = try await communityRepository.editCommunity(withId: draft.identifier!, options: builder)
                self.isEditorLoading = false
                Log.add(info: "Community updated successfully")
                
                Log.add(info: "Community Display Name: \(String(describing: community.displayName))")
                Log.add(info: "Community Avatar: \(String(describing: community.avatar?.attributes))")
                
                completion(true, nil)
            } catch {
                self.isEditorLoading = false
                Log.add(info: "Error while updating community")
                completion(false, error)
            }
        }
    }
    
    func deleteCommunity(completion: @escaping (Bool, Error?) -> Void) {
        communityRepository.deleteCommunity(withId: draft.identifier!) { [weak self] (success, error) in
            self?.isEditorLoading = false
            completion(success, error)
        }
    }
    
    private lazy var _datasource: CommunityEditorDatasource = {
        return self
    }()
    
    func getAmityImageData(completion: @escaping ((AmityImageData?) -> Void) ) {
        if let imageData = self.selectedImage?.pngData(), let image = UIImage(data: imageData) {
            self.uploadAvatar(image: image) { amityImageData in
                completion(amityImageData)
            }
        } else {
            completion(nil)
        }
    }
    
    var datasource: CommunityEditorDatasource {
        get {
            return _datasource
        }
        set {
            _datasource = newValue
        }
    }
    
    var action: CommunityEditorAction { return self}

}
