//
//  CreateStoryView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 28/11/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AVKit
import AmitySDK

struct CreateStoryView: View {
    @State private var showSheet = false
    @State private var selectedType: AmityStoryType = .image
    @State private var hyperlinkText = ""
    @State private var customLinkText = ""
    @State private var choosenTypeText = "Choose image or video"
    @State private var showAlert = false
    @State private var showChoseImageDialog = false
    @State private var showChoseVideoDialog = false
    @State private var alertText = ""
    @State private var newlyCreatedStory = ""
    @State private var isButtonEnable = true
    @State private var isImportingFile = false
    
    @State private var mediaURL: URL? = nil
    @State private var temporaryURL: URL? = nil
    @State private var mediaTypes: [String] = []
    @State private var sourceType: UIImagePickerController.SourceType = .photoLibrary
    @State private var imageDisplayMode: AmityStoryImageDisplayMode = .fit
    
    @Environment(\.presentationMode) private var presentationMode
    
    var feedManager: StoryManager
    var communityId: String
    
    var body: some View {
        VStack(spacing: 20) {
            Text("\(choosenTypeText)")
            HStack {
                
                Text("Choose image")
                    .frame(maxWidth: .infinity, minHeight: 50)
                    .background(.blue)
                    .cornerRadius(16)
                    .foregroundColor(.white)
                    .padding(.horizontal, 20)
                    .onTapGesture {
                        showChoseImageDialog.toggle()
                    }
                    .confirmationDialog("Select an option", isPresented: $showChoseImageDialog) {
                        Button("Camera") {
                            
                            mediaTypes = [UTType.image.identifier]
                            selectedType = .image
                            sourceType = .camera
                            showSheet = true
                        }

                        Button("Gallery") {
                            mediaTypes = [UTType.image.identifier]
                            selectedType = .image
                            sourceType = .photoLibrary
                            showSheet = true
                        }

                    }
                Text("Choose video")
                    .frame(maxWidth: .infinity, minHeight: 50)
                    .background(.blue)
                    .cornerRadius(16)
                    .foregroundColor(.white)
                    .padding(.horizontal, 20)
                    .onTapGesture {
                        showChoseVideoDialog.toggle()
                    }
                    .confirmationDialog("Select an option", isPresented: $showChoseVideoDialog) {
                        Button("Camera") {
                            
                            mediaTypes = [UTType.movie.identifier]
                            selectedType = .video
                            sourceType = .camera
                            showSheet = true
                        }

                        Button("Gallery") {
                            mediaTypes = [UTType.movie.identifier]
                            selectedType = .video
                            sourceType = .photoLibrary
                            showSheet = true
                        }
                        
                        Button("File") {
                            isImportingFile = true
                            selectedType = .video
                        }
                    }
            }
            
            .fileImporter(isPresented: $isImportingFile, allowedContentTypes: [.content, .archive, .item]) { result in
                switch result {
                case .success(let url):
                    createTemporyURL(pickedURL: url)
                    choosenTypeText = "\(selectedType) selected"
                default:
                    break
                }
            }
            
            HStack {
                Text("Image Display Mode")
                    .font(.caption)
                Spacer()
                Picker("Image Aspect", selection: $imageDisplayMode) {
                    Text("Fit").tag(AmityStoryImageDisplayMode.fit)
                    Text("Fill").tag(AmityStoryImageDisplayMode.fill)
                }
                .pickerStyle(.segmented)
            }
            
            VStack(spacing: 10) {
                Text("Insert a link (optional)")
                TextField("Link", text: $hyperlinkText)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
                
                TextField("Custom text", text: $customLinkText)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
            }
            
            Button(action: {
                Task {
                    do {
                        isButtonEnable = false
                        var hyperlink: AmityHyperLinkItem? = nil
                        if hyperlinkText != "" {
                            hyperlink = AmityHyperLinkItem(url: hyperlinkText, customText: customLinkText != "" ? customLinkText : nil )
                        }
                        if let story = try await feedManager.createStory(mediaURL: mediaURL, type: selectedType, hyperlink: hyperlink, communityId: communityId, metadata: nil, imageDisplayMode: imageDisplayMode) {
                            
                            alertText = "Story successfully created"
                            newlyCreatedStory = story.storyId
                        } else {
                            alertText = "No URL or community Id"
                        }
                        showAlert = true
                        isButtonEnable = true
                        removeTemporaryURL(url: temporaryURL)
                        temporaryURL = nil
                        
                    } catch {
                        alertText = error.localizedDescription
                        showAlert = true
                        isButtonEnable = true
                        
                    }
                }
            }) {
                Text("Create Story")
                    .padding()
                    .foregroundColor(.white)
                    .background(Color.blue)
                    .cornerRadius(8)
            }
            .cornerRadius(16)
            .disabled(!isButtonEnable)
            .alert(isPresented: $showAlert) {
                Alert(
                    title: Text(alertText),
                    dismissButton: .default(Text("OK")) {
                        presentationMode.wrappedValue.dismiss()
                    }
                )
                
            }
            
            if !isButtonEnable {
                ProgressView("Creating story...")
                    .padding()
            }
            
        }
        .padding()
        .frame(maxWidth: .infinity, maxHeight: .infinity, alignment: .topLeading)
        .sheet(isPresented: $showSheet, onDismiss: {
            choosenTypeText = "\(selectedType) selected"
        }) {
            StoryImagePicker(mediaTypes: $mediaTypes, selectedURL: $mediaURL, sourceType: $sourceType)
        }
    }
    
    private func removeTemporaryURL(url: URL?) {
        guard let url else { return }
        if FileManager.default.fileExists(atPath: url.path) {
            do {
                try FileManager.default.removeItem(at: url)
                print("Remove item at", url)
            } catch {
                print("Error deleting temporary file: \(error)")
            }
        }
    }
    
    private func createTemporyURL(pickedURL: URL) {
        
        if pickedURL.startAccessingSecurityScopedResource() {
            defer {
                pickedURL.stopAccessingSecurityScopedResource()
            }
            let temporaryURL = FileManager.default.temporaryDirectory.appendingPathComponent(pickedURL.lastPathComponent)
            removeTemporaryURL(url: temporaryURL)
            do {
                try FileManager.default.copyItem(at: pickedURL, to: temporaryURL)
                mediaURL = temporaryURL
                self.temporaryURL = temporaryURL
            } catch {
                print("Error Copying File", error)
            }
            
        }
    }
}


struct StoryImagePicker: UIViewControllerRepresentable {
    
    @Environment(\.presentationMode) private var presentationMode
    
    @Binding var mediaTypes: [String]
    @Binding var selectedURL: URL?
    @Binding var sourceType: UIImagePickerController.SourceType
    
    
    func makeUIViewController(context: UIViewControllerRepresentableContext<StoryImagePicker>) -> UIImagePickerController {
        
        let imagePicker = UIImagePickerController()
        imagePicker.allowsEditing = false
        imagePicker.sourceType = sourceType
        imagePicker.delegate = context.coordinator
        imagePicker.mediaTypes = mediaTypes
        imagePicker.videoQuality = .typeHigh
        imagePicker.videoExportPreset = AVAssetExportPresetPassthrough
        
        return imagePicker
    }
    
    func updateUIViewController(_ uiViewController: UIImagePickerController, context: UIViewControllerRepresentableContext<StoryImagePicker>) {
        
    }
    
    func makeCoordinator() -> Coordinator {
        Coordinator(self)
    }
    
    final class Coordinator: NSObject, UIImagePickerControllerDelegate, UINavigationControllerDelegate {
        
        var parent: StoryImagePicker
        
        init(_ parent: StoryImagePicker) {
            self.parent = parent
        }
        
        func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey: Any]) {
            
            if let mediaType = info[.mediaType] as? String {
                switch mediaType {
                case UTType.image.identifier:
                    
                    if let url = info[.imageURL] as? URL {
                        parent.selectedURL = url
                        parent.presentationMode.wrappedValue.dismiss()
                    } else if let image = info[.originalImage] as? UIImage {
                        if let imageData = image.jpegData(compressionQuality: 1.0) {
                            let temporaryDirectory = FileManager.default.temporaryDirectory
                            let temporaryImageURL = temporaryDirectory.appendingPathComponent("tempImage.jpg")

                            do {
                                try imageData.write(to: temporaryImageURL)
                                // Handle the temporary image URL here
                                print("Temporary Image URL: \(temporaryImageURL)")
                                parent.selectedURL = temporaryImageURL
                                parent.presentationMode.wrappedValue.dismiss()
                            } catch {
                                print("Error saving image: \(error)")
                            }
                        }
                    }
                case UTType.movie.identifier:
                    if let url = info[.mediaURL] as? URL {
                        parent.selectedURL = url
                        parent.presentationMode.wrappedValue.dismiss()
                    }
                default:
                    break
                }
            }
            
        }
        
    }
}
